@@ -4,8 +4,11 @@ module Main
   ) where
 
 import Control.Concurrent (threadDelay)
-import Control.Monad (forM_)
+import Control.Monad (forM_, replicateM_)
 import System.IO (hFlush, stdout)
+import Text.Printf(printf)
+
+import Data.Colour.SRGB (sRGB24)
 
 import System.Console.ANSI
 
@@ -16,10 +19,12 @@ examples = [ cursorMovementExample
            , saveRestoreCursorExample
            , clearExample
            , scrollExample
-           , sgrExample
+           , sgrColorExample
+           , sgrOtherExample
            , cursorVisibilityExample
            , titleExample
            , getCursorPositionExample
+           , getTerminalSizeExample
            ]
 
 main :: IO ()
@@ -215,8 +220,8 @@ scrollExample = do
   -- Line Two
   -- Line Three
 
-sgrExample :: IO ()
-sgrExample = do
+sgrColorExample :: IO ()
+sgrColorExample = do
   let colors = enumFromTo minBound maxBound :: [Color]
   forM_ [Foreground, Background] $ \layer ->  do
     forM_ [Dull, Vivid] $ \intensity -> do
@@ -226,8 +231,90 @@ sgrExample = do
         setSGR [SetColor layer intensity color]
         putStrLn (show color)
       pause
-  -- All the colors, 4 times in sequence
+  -- The ANSI eight standard colors, 4 times in sequence (two layers and two
+  -- intensities)
 
+  resetScreen
+  putStrLn "True color (24 bit color depth)"
+  putStrLn "-------------------------------"
+  putStrLn ""
+  setSGR [SetRGBColor Foreground $ sRGB24 0 0 0]
+  forM_ [0 .. 23] $ \row -> do
+    forM_ [0 .. 47] $ \col -> do
+      let r = row * 11
+          g = 255 - r
+          b = col * 5
+      setSGR [SetRGBColor Background $ sRGB24 r g b]
+      putStr "-"
+    putStrLn ""
+  replicateM_ 5 pause
+  -- True colors, a swatch of 24 rows and 48 columns
+
+  resetScreen
+  putStrLn "A 256-color palette"
+  putStrLn "-------------------"
+  putStrLn ""
+
+  -- First 16 colors ('system' colors in xterm protocol), in a row
+  --
+  -- 0 0 1 1 2 2 3 3 4 4 5 5 6 6 7 7 8 8 9 9 A A B B C C D D E E F F
+  forM_ [Dull .. Vivid] $ \intensity -> do
+    forM_ [Black .. White] $ \color -> do
+    let i = fromEnum intensity * 8 + fromEnum color
+        eol = i == 15
+    setSGR [SetPaletteColor Background $ xtermSystem intensity color]
+    setSGR [SetPaletteColor Foreground $ xtermSystem Dull Black]
+    printf "%X " i
+    setSGR [SetPaletteColor Foreground $ xtermSystem Vivid White]
+    printf "%X" i
+    if eol
+      then putStrLn ""
+      else do
+        setSGR [Reset]
+        putStr " "
+  putStrLn ""
+
+  -- Next 216 colors (6 level RGB in xterm protocol), in 12 rows of 18
+  --
+  -- 000 001 002 003 004 005 010 011 012 013 014 015 020 021 022 023 024 025
+  -- 030 031 032 033 034 035 040 041 042 043 044 045 050 051 052 053 054 055
+  -- 100 101 102 103 104 105 110 111 112 113 114 115 120 121 122 123 124 125
+  -- ... and so on ...
+  forM_ [0 .. 5] $ \r -> do
+    forM_ [0 .. 5] $ \g -> do
+      forM_ [0 .. 5] $ \b -> do
+        let i = 16 + b + g * 6 + r * 36
+            eol = i `mod` 18 == 15
+            r' = (r + 3) `mod` 6
+            g' = (g + 3) `mod` 6
+            b' = (b + 3) `mod` 6
+        setSGR [SetPaletteColor Foreground $ xterm6LevelRGB r' g' b']
+        setSGR [SetPaletteColor Background $ xterm6LevelRGB r g b]
+        putStr $ show r ++ show g ++ show b
+        if eol
+          then putStrLn ""
+          else do
+            setSGR [Reset]
+            putStr " "
+  putStrLn ""
+
+  -- Final 24 colors (24 levels of gray in xterm protocol), in two rows
+  --
+  --   0   1   2   3   4   5   6   7   8   9  10  11
+  --  12  13  14  15  16  17  18  19  20  21  22  23
+  forM_ [0 .. 23] $ \y -> do
+    setSGR [SetPaletteColor Foreground $ xterm24LevelGray $ (y + 12) `mod` 24]
+    setSGR [SetPaletteColor Background $ xterm24LevelGray y]
+    printf "%3d" y
+    if y == 11
+      then putStrLn ""
+      else do
+        setSGR [Reset]
+        putStr " "
+  replicateM_ 5 pause
+
+sgrOtherExample :: IO ()
+sgrOtherExample = do
   let named_styles = [ (SetConsoleIntensity BoldIntensity, "Bold")
                      , (SetConsoleIntensity FaintIntensity, "Faint")
                      , (SetConsoleIntensity NormalIntensity, "Normal")
@@ -301,15 +388,25 @@ getCursorPositionExample = do
   --          11111111112222222222
   -- 12345678901234567890123456789
   -- Report cursor position here:|
-  result <- getCursorPosition0
+  result <- getCursorPosition
   putStrLn " (3rd row, 29th column) to stdin, as CSI 3 ; 29 R.\n"
   case result of
     Just (row, col) -> putStrLn $ "The cursor was at row number " ++
       show (row + 1) ++ " and column number " ++ show (col + 1) ++ ".\n"
     Nothing -> putStrLn "Error: unable to get the cursor position\n"
-  pause
+  replicateM_ 3 pause
   --          11111111112222222222
   -- 12345678901234567890123456789
   -- Report cursor position here: (3rd row, 29th column) to stdin, as CSI 3 ; 29 R.
   --
   -- The cursor was at row number 3 and column number 29.
+
+getTerminalSizeExample :: IO ()
+getTerminalSizeExample = do
+  result <- getTerminalSize
+  case result of
+    Just (h, w) -> putStrLn $ "The size of the terminal is " ++ show h ++
+      " rows by " ++ show w ++ " columns.\n"
+    Nothing -> putStrLn "Error: unable to get the terminal size\n"
+  pause
+  -- The size of the terminal is 25 rows by 80 columns.
