@@ -1,3 +1,4 @@
+#include "Common-Safe-Haskell.hs"
 {-# OPTIONS_HADDOCK hide #-}
 
 module System.Console.ANSI.Windows
@@ -8,7 +9,7 @@ module System.Console.ANSI.Windows
 #include "Exports-Include.hs"
   ) where
 
-import System.IO (Handle, stdout)
+import System.IO (Handle)
 
 import System.Console.ANSI.Types
 import qualified System.Console.ANSI.Unix as U
@@ -190,7 +191,6 @@ hSupportsANSIWithoutEmulation = E.hSupportsANSIWithoutEmulation
 -- (See Common-Include.hs for Haddock documentation)
 getReportedCursorPosition = E.getReportedCursorPosition
 
--- getCursorPosition :: IO (Maybe (Int, Int))
+-- hGetCursorPosition :: Handle -> IO (Maybe (Int, Int))
 -- (See Common-Include.hs for Haddock documentation)
-{-# DEPRECATED getCursorPosition "Use getCursorPosition0 instead." #-}
-getCursorPosition = E.getCursorPosition
+hGetCursorPosition = E.hGetCursorPosition
