@@ -1,3 +1,4 @@
+{-# LANGUAGE Trustworthy #-}
 {-# OPTIONS_HADDOCK hide #-}
 
 module System.Console.ANSI.Windows.Emulator
@@ -13,8 +14,7 @@ import Data.IORef (IORef, newIORef, readIORef, writeIORef)
 import Data.List (foldl', minimumBy)
 import Data.Maybe (mapMaybe)
 import qualified Data.Map.Strict as Map (Map, empty, insert, lookup)
-import System.Environment (getEnvironment)
-import System.IO (Handle, hFlush, hIsTerminalDevice, stdin, stdout)
+import System.IO (Handle, hIsTerminalDevice, stdin)
 import System.IO.Unsafe (unsafePerformIO)
 import Text.ParserCombinators.ReadP (readP_to_S)
 
@@ -77,9 +77,15 @@ adjustCursorPosition :: HANDLE
 adjustCursorPosition handle change_x change_y = do
   screen_buffer_info <- getConsoleScreenBufferInfo handle
   let window = csbi_window screen_buffer_info
+      l = rect_left window
+      t = rect_top window
+      r = rect_right window
+      b = rect_bottom window
       (COORD x y) = csbi_cursor_position screen_buffer_info
-      cursor_pos'= COORD (change_x (rect_left window) x)
-                         (change_y (rect_top window) y)
+      clamp mn mx = max mn . min mx
+      x' = clamp l r (change_x l x)
+      y' = clamp t b (change_y t y)
+      cursor_pos' = COORD x' y'
   setConsoleCursorPosition handle cursor_pos'
 
 hCursorUp h n
@@ -263,6 +269,12 @@ swapForegroundBackgroundColors attribute
 applyANSISGRToAttribute :: WORD -> SGR -> WORD -> WORD
 applyANSISGRToAttribute def sgr attribute = case sgr of
   Reset -> def
+  SetDefaultColor Foreground ->
+    (attribute .&. complement fOREGROUND_INTENSE_WHITE) .|.
+    (def .&. fOREGROUND_INTENSE_WHITE)
+  SetDefaultColor Background ->
+    (attribute .&. complement bACKGROUND_INTENSE_WHITE) .|.
+    (def .&. bACKGROUND_INTENSE_WHITE)
   SetConsoleIntensity intensity -> case intensity of
     BoldIntensity   -> attribute .|. iNTENSITY
     FaintIntensity  -> attribute .&. (complement iNTENSITY) -- Not supported
@@ -309,6 +321,7 @@ applyANSISGRToAttribute def sgr attribute = case sgr of
           Dull  -> attribute .&. complement bACKGROUND_INTENSITY
           Vivid -> attribute .|. bACKGROUND_INTENSITY
     in applyBackgroundANSIColorToAttribute aNSIColor attribute'
+  SetPaletteColor _ _ -> attribute  -- Not supported
  where
   iNTENSITY = fOREGROUND_INTENSITY
 
@@ -470,24 +483,27 @@ getReportedCursorPosition
       isKeyDown = keyEventKeyDown keyEventRecord
       isKeyDownEvent = eventType == 1 && isKeyDown
 
--- getCursorPosition :: IO (Maybe (Int, Int))
+-- hGetCursorPosition :: Handle -> IO (Maybe (Int, Int))
 -- (See Common-Include.hs for Haddock documentation)
-getCursorPosition = CE.catch getCursorPosition' getCPExceptionHandler
+hGetCursorPosition h = fmap to0base <$> getCursorPosition'
  where
-  getCursorPosition' = do
-    withHandleToHANDLE stdin flush -- Flush the console input buffer
-    reportCursorPosition
-    hFlush stdout -- ensure the report cursor position code is sent to the
-                  -- operating system
-    input <- getReportedCursorPosition
-    case readP_to_S cursorPosition input of
-      [] -> return Nothing
-      [((row, col),_)] -> return $ Just (row, col)
-      (_:_) -> return Nothing
+  to0base (row, col) = (row - 1, col - 1)
+  getCursorPosition' = CE.catch getCursorPosition'' getCPExceptionHandler
    where
-    flush hdl = do
-      n <- getNumberOfConsoleInputEvents hdl
-      unless (n == 0) (void $ readConsoleInput hdl n)
+    getCursorPosition'' = do
+      withHandleToHANDLE stdin flush -- Flush the console input buffer
+      hReportCursorPosition h
+      hFlush h -- ensure the report cursor position code is sent to the
+               -- operating system
+      input <- getReportedCursorPosition
+      case readP_to_S cursorPosition input of
+        [] -> return Nothing
+        [((row, col),_)] -> return $ Just (row, col)
+        (_:_) -> return Nothing
+     where
+      flush hdl = do
+        n <- getNumberOfConsoleInputEvents hdl
+        unless (n == 0) (void $ readConsoleInput hdl n)
 
 getCPExceptionHandler :: IOException -> IO a
 getCPExceptionHandler e = error msg
