@@ -107,12 +107,18 @@
 
     -- * Checking if handle supports ANSI (not portable: GHC only)
   , hSupportsANSI
+  , hSupportsANSIColor
   , hSupportsANSIWithoutEmulation
 
     -- * Getting the cursor position
-  , getCursorPosition0
+  , getCursorPosition
+  , hGetCursorPosition
   , getReportedCursorPosition
   , cursorPosition
 
+    -- * Getting the terminal size
+  , getTerminalSize
+  , hGetTerminalSize
+
     -- * Deprecated
-  , getCursorPosition
+  , getCursorPosition0
